clear all
dynare anscho
%load anscho_results;


% Restricted parameters
restr_names = []; %[{'rhozg'};{'rhogz'}];
restr_indx = [];
nrestr = size(restr_names,1);
for i=1:nrestr
    restr_indx = [restr_indx; strmatch(restr_names(i,:),M_.param_names,'exact')];
end
restr_values = M_.params(restr_indx);

% Parameter values
x = zeros(M_.param_nbr,1);
for i=1:M_.param_nbr
    x(i) = M_.params(i);
end

% Names and values of unrestricted parameters
theta_names = cellstr(M_.param_names);
theta_names(restr_indx,:)=[];
x(restr_indx,:)=[];

% Number of unrestricted parameters
ntheta = size(x,1);

% Collecting info about parameters
params_.theta_names     = theta_names;
params_.ntheta          = ntheta;
params_.restr_names     = restr_names;
params_.restr_values    = restr_values;
params_.nrestr          = nrestr;


[q n] = size(oo_.dr.ghx); %n is number of states
q = q-n; %number of non-state variables
k = 3; %number of observables (equal to number of shocks)
h = (k-1)*k/2; %number of free elements in orthogonal matrix
model_.n     = n;
model_.q     = q;
model_.k     = k;
model_.h     = h;

% Model solution
A = oo_.dr.ghx(1:n,1:n);
B = oo_.dr.ghu(1:n,1:k);
F1 = oo_.dr.ghx(n+1:end,:);
G1 = oo_.dr.ghu(n+1:end,1:k);

H=[zeros(k,2) eye(k,k)];
Hs= H(:,1:n);
Hp= H(:,n+1:end);

C = Hs*A+Hp*F1;
D = Hs*B+Hp*G1;

% Collecting info about model
model_.A     = A;
model_.B     = B;
model_.C     = C;
model_.D     = D;
model_.F1    = F1;
model_.G1    = G1;
model_.Hs    = Hs;
model_.Hp    = Hp;

% Vectorizing F matrix (start from F1)
xF= reshape(F1,q*n,1);

% Vectorizing G matrix (start from G1)
xG= reshape(G1,q*k,1);

% Vectorizing T matrix (start from I)
T = eye(n);
xT= reshape(T,n*n,1);

% Free elements in H matrix (start from I)
xH= zeros(h,1);

% Original model solution
xorig = [x; xF; xG; xT; xH];

nall = size(xorig,1);

%% Checking global identification

% Maximum number of checks 
M = 1000;

% Initialization
count = 0;
x = xorig;

% Calibrating the scale of randomizing distribution for initial values
rand_params = zeros(size(x)); 
rand_params(1:ntheta) = ones(ntheta,1); % deep parameters
rand_params(ntheta+1:end) = 0.01*ones(nall-ntheta,1); % remaining unknowns (F, G, T, U)

options = optimset('TolFun',1e-20,'TolX',1e-20,'Algorithm','trust-region-reflective');

for i=1:M
    count=count+1;
    disp(count);
    exitflag = 0;
    while exitflag<1
        x0 = xorig + rand_params.*randn(size(xorig,1),1);
        fsys9 = @(x)sys9(x,model_,params_);
        [x fval exitflag output] = fsolve(fsys9,x0,options);
    end
    if any(abs(xorig-x)>1e-5)
        disp(['Model not globally identified. Number of performed evaluations: ' num2str(count)]);
        break
    end
end

if count == M
    disp(['Global identification failure not detected after ' num2str(M) ' evaluations']);
end

% Displaying theta_bar
[F T U F2 G2 Gam0 Gam1 Gam2 Gam3] = sys9(x,model_,params_);
for i=1:ntheta
    disp([theta_names{i},'=',num2str(x(i),12),';']);
end


%% Checking Blanchard-Kahn conditions

GGam1 = [Gam1 zeros(n+q,n);
        zeros(n,n+q) eye(n,n)];
GGam0 = [Gam0 -Gam2;
        zeros(n,n+q) eye(n,n)];

    if sum(abs(eig(GGam0,GGam1))>1) < 4 
    disp('Solution for theta_bar not unique (indeterminacy)!');
end
