function [F T U F2 G2 Gam0 Gam1 Gam2 Gam3] = sys9par(x,model_,params_)

%global model_ params_


ntheta  = params_.ntheta;

n       = model_.n;
k       = model_.k;
q       = model_.q;
h       = model_.h;

A       = model_.A;
B       = model_.B;
C       = model_.C;
D       = model_.D;
F1      = model_.F1;
G1      = model_.G1;
Hs      = model_.Hs;
Hp      = model_.Hp;

for i = 1:length(params_.theta_names)
  par.(params_.theta_names{i}) = x(i);
end

for i = 1:length(params_.restr_names)
  par.(params_.restr_names{i}) = params_.restr_values(i);
end


% Check if parameters are within bounds
chk = (x(1:ntheta) > params_.theta_bounds(:,1)) & (x(1:ntheta) < params_.theta_bounds(:,2));
dimF = (n+q)*n + (n+q)*k + k*n + k*k;
if any(~chk)
    F = 1000*ones(dimF,1);
    return
end


% Var order: s1 s2 p

Gam0 = [1 0 0;
        0 1 par.alpha2;
        par.alpha1^2*par.alpha2-1 0 1];

Gam1 = [0 0 0;
        0 0 0;
        0 0 par.alpha2];
    
Gam2 = [par.alpha1^2 0;
        1-par.alpha1^2 1-par.alpha1^2;
        0 0];
    
Gam3 = [1;
        0;
        0];
    
Gam0s= Gam0(:,1:n);
Gam0p= Gam0(:,n+1:end);
Gam1s= Gam1(:,1:n);
Gam1p= Gam1(:,n+1:end);

F2= reshape(x(ntheta+1:end-q*k-n*n-h,1),q,n);

G2= reshape(x(ntheta+1+q*n:end-n*n-h,1),q,k);

T = reshape(x(ntheta+1+q*n+q*k:end-h,1),n,n);

U = zeros(k,k);

cnt=1;
for i=2:k
    for j=1:(i-1)
        U(i,j)=x(end-h+cnt);
        cnt=cnt+1;
    end
end
U = U - U';
U = 2 * inv(U + eye(k)) - eye(k); %creating orthogonal matrix from skew-symmetric matrix


F= [vec((Gam0s - Gam1s*T*A*inv(T) - Gam1p*F2)*T*A*inv(T) + Gam0p*F2 - Gam2);
    vec((Gam1s*T*A*inv(T) + Gam1p*F2 - Gam0s)*T*B*U + Gam3 - Gam0p*G2);
    vec(C*inv(T) - Hs*T*A*inv(T) - Hp*F2);
    vec(D*U - Hs*T*B*U - Hp*G2)];

